#!/usr/bin/env python3
#SBATCH --partition xeon24
#SBATCH --ntasks 24
#SBATCH --time 6:0:0
import os
from ase.collections import g2
from ase.optimize import (BFGS, GPMin, BFGSLineSearch, GoodOldQuasiNewton,
                          FIRE, LBFGS, LBFGSLineSearch)
from ase.optimize.sciopt import SciPyFminCG, SciPyFminBFGS
from gpaw import GPAW, Mixer
from gpaw.mpi import world
from contextlib import contextmanager
from tasks import itasks
from gpaw.mpi import SerialCommunicator

#world = SerialCommunicator()

def optimize(g2name, optimizer):
    atoms = g2[g2name]
    atoms.center(vacuum=3.5)
    calc = GPAW(mode='lcao', basis='dzp', h=0.18,
                mixer=Mixer(0.4, 7, 50.0),
                convergence=dict(density=1e-5),
                xc='oldLDA', txt='gpaw.txt')
    atoms.rattle(stdev=0.1, seed=1234 + g2.names.index(g2name))
    atoms.calc = calc

    kwargs = dict(trajectory='opt.traj',
                  logfile='opt.log')
    if optimizer == 'bfgs':
        opt = BFGS(atoms, maxstep=0.2, **kwargs)
    elif optimizer == 'bfgs-bad':
        opt = BFGS(atoms, **kwargs)
    elif optimizer == 'sciopt-cg':
        opt = SciPyFminCG(atoms, **kwargs)
    elif optimizer == 'sciopt-bfgs':
        opt = SciPyFminBFGS(atoms, **kwargs)
    elif optimizer == 'bfgsls':
        opt = BFGSLineSearch(atoms, **kwargs)
    elif optimizer == 'lbfgsls':
        opt = LBFGSLineSearch(atoms, **kwargs)
    elif optimizer == 'lbfgs':
        opt = LBFGS(atoms, **kwargs)
    elif optimizer == 'oldqn':
        opt = GoodOldQuasiNewton(atoms, **kwargs)
    elif optimizer == 'fire':
        opt = FIRE(atoms, **kwargs)
    elif optimizer == 'gpmin':
        opt = GPMin(atoms, **kwargs)
    elif optimizer == 'gpmin-smallstep':
        opt = GPMin(atoms, scale=0.2, **kwargs)
    else:
        raise ValueError('strange optimizer {}'.format(optimizer))

    opt.run(fmax=0.03)

workdir = os.getcwd()

i = 0
for task in itasks():
    xid = task.xid
    dirname = task.dirname
    g2name, optimizer = task.args

    exists = 0
    if world.rank == 0:
        exists = int(os.path.exists(dirname))
        if not exists:
            os.mkdir(dirname)
    exists = world.sum(exists)
    if exists:
        continue

    def writestatus(msg):
        with open('status.txt', 'w') as fd:
            print(msg, file=fd)

    try:
        writestatus('started')
        os.chdir(dirname)
        optimize(g2name, optimizer)
    except Exception:
        writestatus('failed')
        import traceback
        msg = traceback.format_exc()
        with open('stacktrace.txt', 'w') as fd:
            print(msg, file=fd)
        continue
    else:
        writestatus('done')
    finally:
        os.chdir(workdir)

    i += 1
    if i > 128:
        break
